from cozmo_fsm import *
import math

global visited_doors
visited_doors = []

#Determines if there is an unvisited door in the robots world view
class LookForDoor(StateNode):
	def start(self, event =  None):
		global visited_doors, next_door

		if self.running: return
		super().start(event)

		#Gets all of the objects the robot knows the location of
		current_world = robot.world.world_map.objects
		
		#Sees if there are any doors among them
		door_list = []
		for i in current_world:
			i_string = str(i)
			if "Doorway" in i_string:
				door_list.append(int(i_string.split("-")[1]))
		to_visit = []
		
		#Finds doors in view that are unvisited
		for door in door_list:
			if door not in visited_doors:
				to_visit.append(door)

		#Posts failure if there are no unvisited doors in the world view
		if len(to_visit) == 0:
			self.post_failure()
		
		#Posts the numer of the first unvisited if one is found
		else: 
			next_door = to_visit[0]
			visited_doors.append(next_door)
			self.post_data(next_door)


#Explores the environment by going through doors
#Assumes that the environment is compact enough that the
#robot does not need to arbitrarily drive around infinitely to find doors
class Exploration(StateMachineProgram):
	
 def setup(self):
     """
    		
    		#Goes through the unvisited door if one is found, keep looking if not
    		look: LookForDoor()
    		look =F=> turn_and_look
    		look =D=> through_door
    
    		#If the robot is 100mm back after coming through a perpendicular door
    		#it will be able to get 2 Aruco markers for the door in its view and localize it
    		turn_and_look: Turn(90) =C=> DriveForward(-100) =C=> DriveForward(100) =C=> look
    
    		#Gets a little bit away from the door after going though to make it easier to see others
    		through_door: DoorPass() =C=> DriveForward(50) =C=> look
    
     """
     
     # Code generated by genfsm on Wed Feb 19 19:02:02 2020:
     
     look = LookForDoor() .set_name("look") .set_parent(self)
     turn_and_look = Turn(90) .set_name("turn_and_look") .set_parent(self)
     driveforward1 = DriveForward(-100) .set_name("driveforward1") .set_parent(self)
     driveforward2 = DriveForward(100) .set_name("driveforward2") .set_parent(self)
     through_door = DoorPass() .set_name("through_door") .set_parent(self)
     driveforward3 = DriveForward(50) .set_name("driveforward3") .set_parent(self)
     
     failuretrans1 = FailureTrans() .set_name("failuretrans1")
     failuretrans1 .add_sources(look) .add_destinations(turn_and_look)
     
     datatrans1 = DataTrans() .set_name("datatrans1")
     datatrans1 .add_sources(look) .add_destinations(through_door)
     
     completiontrans1 = CompletionTrans() .set_name("completiontrans1")
     completiontrans1 .add_sources(turn_and_look) .add_destinations(driveforward1)
     
     completiontrans2 = CompletionTrans() .set_name("completiontrans2")
     completiontrans2 .add_sources(driveforward1) .add_destinations(driveforward2)
     
     completiontrans3 = CompletionTrans() .set_name("completiontrans3")
     completiontrans3 .add_sources(driveforward2) .add_destinations(look)
     
     completiontrans4 = CompletionTrans() .set_name("completiontrans4")
     completiontrans4 .add_sources(through_door) .add_destinations(driveforward3)
     
     completiontrans5 = CompletionTrans() .set_name("completiontrans5")
     completiontrans5 .add_sources(driveforward3) .add_destinations(look)
     
     return self
